----------------------------------------------------------------------------------------------------
-- DiscoTelegraphs
--- © 2015 Vim Exe @ Jabbit <narthorn@gmail.com>
--- Original idea by <Enigma>
--
--- DiscoTelegraphs is free software, all files licensed under the GPLv3. See LICENSE for details.

DiscoTelegraphs = {
	name = "DiscoTelegraphs",
	version = {0,4,1},
	settings = {
		bEnabled = true,
		bPvP = false,
		fSpeed = 0.1,
	},
	tColors = {
		{255, 255, 255, 100}, -- white
		{255,   0,   0, 100}, -- red
		{255, 255,   0, 100}, -- yellow
		{  0, 255, 255, 100}, -- cyan
		{255,   0, 255, 100}, -- purple
		{  0,   0, 255, 100}, -- blue
	},
	nCurrentColorId = 1,
}

function DiscoTelegraphs:OnLoad()
	Event_FireGenericEvent("OneVersion_ReportAddonInfo", self.name, unpack(self.version))
	Apollo.RegisterSlashCommand("disco", "OnSlashCommand", self)
	self.tmrInit = ApolloTimer.Create(0, false, 'Init', self) -- Wait 0s to let OnRestore fire
end

function DiscoTelegraphs:OnSave(eLevel)
	if eLevel ~= GameLib.CodeEnumAddonSaveLevel.Character then return end
	if self.tmrDisco then self:RestoreDefaults() end
	return {version = self.version, settings = self.settings}
end

function DiscoTelegraphs:OnRestore(eLevel, tData)
	if eLevel ~= GameLib.CodeEnumAddonSaveLevel.Character then return end
	if not tData.version or tData.version[2] < 3 then
		tData.settings = nil
	elseif tData.version[2] < 4 then
		tData.settings.bPvP = false
	end

	self.settings = tData.settings or self.settings
end

function DiscoTelegraphs:Init()
	self:SaveDefaults()
	self:OnCharacterCreated()
end

function DiscoTelegraphs:OnCharacterCreated()
	local unit = GameLib.GetPlayerUnit()
	if not unit then Apollo.RegisterEventHandler("CharacterCreated", "OnCharacterCreated", self); return end
	Apollo.RemoveEventHandler("CharacterCreated")
	self:Refresh()
end

function DiscoTelegraphs:Activate()
	if not self.tmrDisco then
		self:SaveDefaults()
		Apollo.SetConsoleVariable("spell.telegraphColorSet", 4)
		local strTimer = (self.settings.bPvP and MatchingGame.IsInPVPGame()) and "OnPvPTimer" or "OnTimer"
		self.tmrDisco = ApolloTimer.Create(self.settings.fSpeed, true, strTimer, self)
	end
end

function DiscoTelegraphs:Deactivate()
	if self.tmrDisco then
		self.tmrDisco:Stop()
		self.tmrDisco = nil
		self:RestoreDefaults()
		GameLib.RefreshCustomTelegraphColors()
	end
end

function DiscoTelegraphs:OnChangeWorld()
	-- If the player changes zone while in combat, no out of combat event will be fired,
	-- so we have to deactivate manually in that case
	self:Deactivate()

	if self.settings.bPvP and MatchingGame.IsInPVPGame() then
		Apollo.RemoveEventHandler("UnitEnteredCombat", self)
		self:Activate()
	else
		Apollo.RegisterEventHandler("UnitEnteredCombat", "OnCombat", self)
	end
end

function DiscoTelegraphs:OnCombat(unit, bCombat)
	if unit and unit:IsThePlayer() then
		if bCombat then self:Activate() else self:Deactivate() end
	end
end

function DiscoTelegraphs:OnTimer()
	self:SetColor(self.tColors[self.nCurrentColorId])
	GameLib.RefreshCustomTelegraphColors()
	self.nCurrentColorId = (self.nCurrentColorId % #self.tColors) + 1
end

--jesus christ I need to figure out something that's not as retarded as this
function DiscoTelegraphs:OnPvPTimer()
	local tColors = self.tColors[self.nCurrentColorId]
	self:SetColor(tColors)
	self:SetPvPColor(tColors)
	GameLib.RefreshCustomTelegraphColors()
	self.nCurrentColorId = (self.nCurrentColorId % #self.tColors) + 1
end

-- UI

function DiscoTelegraphs:Refresh()
	Apollo.RemoveEventHandler("UnitEnteredCombat", self)
	Apollo.RemoveEventHandler("ChangeWorld", self)
	self:Deactivate()
	local unitPlayer = GameLib.GetPlayerUnit()
	if self.settings.bEnabled then
		Apollo.RegisterEventHandler("ChangeWorld", "OnChangeWorld", self)
		if unitPlayer and unitPlayer:IsInCombat() then self:Activate() end
		if self.settings.bPvP and MatchingGame.IsInPVPGame() then
			self:Activate()
		else
			Apollo.RegisterEventHandler("UnitEnteredCombat", "OnCombat", self)
		end
	end
end

function DiscoTelegraphs:Enable(bEnabled)
	self.settings.bEnabled = bEnabled
	Print(self:GetName().. (bEnabled and " enabled !" or " disabled."))
	self:Refresh()
end

function DiscoTelegraphs:EnablePvP(bEnabled)
	self.settings.bPvP = bEnabled
	Print(self:GetName().. " - PvP telegraphs" .. (bEnabled and " enabled !" or " disabled."))
	self:Refresh()
end

DiscoTelegraphs.SlashCommands = setmetatable({
	on       = function() DiscoTelegraphs:Enable(true) end,
	off      = function() DiscoTelegraphs:Enable(false) end,
	toggle   = function() DiscoTelegraphs:Enable(not DiscoTelegraphs.settings.bEnabled) end,
	pvp      = function(bEnable) DiscoTelegraphs:EnablePvP(not DiscoTelegraphs.settings.bPvP) end,
	speed    = function(fSpeed)

		fSpeed = fSpeed and tonumber(fSpeed)
		if fSpeed and fSpeed ~= 0 then
			DiscoTelegraphs.settings.fSpeed = fSpeed
			Print(DiscoTelegraphs:GetName() .. " - Flash interval set to "..fSpeed.."s.")
			DiscoTelegraphs:Refresh()
		else
			Print(DiscoTelegraphs:GetName() .. " - Flash interval is currently "..DiscoTelegraphs.settings.fSpeed.."s.")
			Print("/disco speed x to set the interval to x seconds.")
		end
	end
},{
	__index = function(t, key) return function()
		Print(DiscoTelegraphs:GetName() .. " - Available commands: ")
		for k,v in pairs(t) do Print("  /disco "..k) end
	end end
})

function DiscoTelegraphs:OnSlashCommand(strCmd, strParams)
	if strParams == "" then	self.SlashCommands.toggle()
	else
		local tArgs = {}
		for arg in strParams:gmatch("[^%s]+") do
			table.insert(tArgs, arg)
		end

		DiscoTelegraphs.SlashCommands[tArgs[1]](unpack(tArgs, 2, #tArgs))
	end
end

-- Utils

function DiscoTelegraphs:SetColor(tColor)
	Apollo.SetConsoleVariable("spell.custom1EnemyNPCDetrimentalTelegraphColorR", tColor[1])
	Apollo.SetConsoleVariable("spell.custom1EnemyNPCDetrimentalTelegraphColorG", tColor[2])
	Apollo.SetConsoleVariable("spell.custom1EnemyNPCDetrimentalTelegraphColorB", tColor[3])
	Apollo.SetConsoleVariable("spell.fillOpacityCustom1_34", tColor[4])
end

function DiscoTelegraphs:SetPvPColor(tColor)
	Apollo.SetConsoleVariable("spell.custom1EnemyPlayerDetrimentalTelegraphColorR", tColor[1])
	Apollo.SetConsoleVariable("spell.custom1EnemyPlayerDetrimentalTelegraphColorG", tColor[2])
	Apollo.SetConsoleVariable("spell.custom1EnemyPlayerDetrimentalTelegraphColorB", tColor[3])
end

function DiscoTelegraphs:SaveDefaults()
	self.nOriginalColorSet = Apollo.GetConsoleVariable("spell.telegraphColorSet")
	self.tOriginalColor = {
		Apollo.GetConsoleVariable("spell.custom1EnemyNPCDetrimentalTelegraphColorR"),
		Apollo.GetConsoleVariable("spell.custom1EnemyNPCDetrimentalTelegraphColorG"),
		Apollo.GetConsoleVariable("spell.custom1EnemyNPCDetrimentalTelegraphColorB"),
		Apollo.GetConsoleVariable("spell.fillOpacityCustom1_34"),
	}
	if self.settings.bPvP then
		self.tOriginalPvPColor = {
			Apollo.GetConsoleVariable("spell.custom1EnemyPlayerDetrimentalTelegraphColorR"),
			Apollo.GetConsoleVariable("spell.custom1EnemyPlayerDetrimentalTelegraphColorG"),
			Apollo.GetConsoleVariable("spell.custom1EnemyPlayerDetrimentalTelegraphColorB"),
		}
	end
end

function DiscoTelegraphs:RestoreDefaults()
	self:SetColor(self.tOriginalColor)
	if self.settings.bPvP then self:SetPvPColor(self.tOriginalPvPColor) end
	Apollo.SetConsoleVariable("spell.telegraphColorSet", self.nOriginalColorSet)
end

function DiscoTelegraphs:GetName()
	return self.name .. " v" .. table.concat(self.version,".")
end

Apollo.RegisterAddon(DiscoTelegraphs)

--- TEST SUITE - because I will always break something on every new build
--
--- Quit game, delete addon save data, restart.
--
--- Enable, go in combat, disable.
--- Enable, go in combat, reloadui, disable.
--- Enable, go in combat, change zone.
--- Enable, go in combat, change speed, disable.
--
--- Enable, change telegraph color, reloadui.
--- Enable, change telegraph color, change zone.
--- Enable, change telegraph color, go in combat, disable.
--- Enable, go in combat, disable, change telegraph color, enable, disable.
--
--- Enter PVP Instance, enable, enable pvp.
--- Enable, enter PVP instance, enable.
--- Enable, enable PvP enter PVP instance, reloadui.
--
